package SlackBuild::Archive::Extractor::HTTP;
use strict;
use warnings;
use parent 'SlackBuild::Archive::Extractor';
use HTML::Parser;
use File::Basename;
use File::Temp;
use File::Spec;
use File::Path qw(make_path);
use Carp;

sub extract {
    my $self = shift;

    my $dst = $self->destdir;

    unless ($self->_html_list($self->tempfile)) {
	return $self->error("$self: bad index file");
    }

    return 0 unless $self->archive->verify;

    return $self->archive->iterate(sub {
	my $file = shift;
	my $subdir = dirname($file);
	my $dir = File::Spec->catfile($dst, $subdir);
	unless (-d $dir) {
	    make_path($dir, { error => \my $err });
	    if (@$err) {
		for my $diag (@$err) {
		    my ($file, $message) = %$diag;
		    if ($file eq '') {
			$self->error("general error: $message");
		    } else {
			$self->error("can't create $file: $message");
		    }
		}
		return 0;
	    }
	}
	my $uri = $self->archive->new_abs($file);
	my (undef,$tmp) = File::Temp::tempfile(DIR => $dir, UNLINK => 1);
	unless ($uri->download($tmp)) {
	    return $self->error("can't download $uri: "
				. $uri->download_status);
	}
	my $destfile = File::Spec->catfile($dst, $file);
	if ($uri->downloaded_html
	    && $self->_html_list($tmp, File::Spec->catfile($subdir, $file))) {
	    ;
	} else {
	    rename $tmp, $destfile
		or $self->error("can't rename $tmp to $destfile: $!");
        }
	return 1;
    });
    return 1;
}
    
sub _html_list {
    my ($self, $filename, $dir) = @_;

    my $p = HTML::Parser->new(
	api_version => 3,
	start_h => [
	    sub {
		my ($attr) = @_;
		return unless $attr->{href};
		my $f = $attr->{href};
		return if $f =~ m{(^([?#/])|(://))};
		$f = File::Spec->catfile($dir, $f) if $dir;
		$self->archive->add_file($f);
	    },
	    'attr'
	]);
    $p->report_tags(qw(a));
    return $p->parse_file($filename);
}   

1;
